#! /usr/bin/env python
'''
Module Name:    WistronVideoFwFlash.py

Description:    Class for Flashing Wistron Video FW Update

Copyright:      Toshiba Confidential

                OCO Source Materials

                Copyright (C) 2012,2013 Toshiba Global Commerce Solutions, Inc.

                The source code for this program is not published
                or otherwise divested of its trade secrets,
                irrespective of what has been deposited with the
                U. S. Copyright Office.

Initial Author: Francisco Liwa

Function:

Notes:

Dependencies:    None

Restrictions:    None

Last Modified: 12 December 2013
'''

import os
import time

from monitorupgrade import TLog as TLOG
from monitorupgrade import Utils as utils

EXIT_SUCCESS = 0
ERROR_SEGFAULT = 139
ERROR_CODE_AIP_SEGFAULT = 12
AIPFLASH_UTIL = '/usr/bin/aipfwupdate'
MUS_LOG_FILE_DIR = '/var/log/toshiba/mus/latest/'
AIP_RECOVERY_FILE = '/opt/toshiba/mus/op_utils/aipfwupdate/recovery.txt'
AIP_FW_UPDATE = '/opt/toshiba/mus/op_utils/aipfwupdate/AipFwUpdate.sh'


##==========================================================================================================
#VideoFwFlashTask_6149_DWU - Class for Flashing DWU Video FW Update
##========================================================================================================== 
class VideoFwFlashTask_6149_DWU:
    fw_path=''
    fw_name=''
    controller=''
    
    def __init__(self,path,name,controller):
        '''
        Constructor
        ''' 
        TLOG.logger.info('VideoFwFlashTask_6149_DWU Constructed')
        self.fw_path = path.strip('\'')
        self.fw_name = name.strip('\'')
        self.controller = controller
    
    
    def recover(self,cmd,path,retry=3):
        ret = 99
        count = 0
        #Try to do internal recovery up to number of retries
        while(ret != 0 and count != retry):    
            txt = 'VideoFwFlashTask_6149_DWU::recover() -----------------RETRY#'+ str(count+1) + '-----------------\n'
            TLOG.logger.info(txt)
            ret = utils.runSubprocess(cmd,'VideoFwFlashTask_6149_DWU',str(path))
            time.sleep(10)
            count += 1
        return ret
                  
    def run(self):
        TLOG.logger.info('VideoFwFlashTask_6149_DWU::run() -----------------START---------------')
        
        aipflashCommand = 'sh ' + AIP_FW_UPDATE +  '  -F ' + self.fw_name
            
        if utils.doesFileExist(AIPFLASH_UTIL) != True:
            
            print '[VideoFwFlashTask_6149_DWU] - No Aipflash Utility found!'
            TLOG.logger.error('VideoFwFlashTask_6149_DWU::run()-No Aipflash Utility found!')
            raise IOError('No Aipflash Utility found!')
        else:
            
            #make recovery text here
            rcv_text = 'AIPRecover:' + ','+  self.fw_path + ',' + self.fw_name  
            f = open(AIP_RECOVERY_FILE,'w')
            f.write(rcv_text)
            f.close()
                       
            print "[VideoFwFlashTask_6149_DWU] - Updating " + self.controller + " firmware..."
            print "[VideoFwFlashTask_6149_DWU] - This will take some time,Please wait ..."
            
            #info = AIP_FW_UPDATE + ' -L'
            #ret = utils.runSubprocess(info,'VideoFwFlashTask_6149_DWU')
            
            time.sleep(10)
            
            ret = utils.runSubprocess(aipflashCommand,'VideoFwFlashTask_6149_DWU',str(self.fw_path) ) 
            
            #Try to recover when there is a segfault cause by libusb-1_0-0 ver 1.0.8 limitation at do_sync() function
            #This only occurs when the combination of 6149 and 4820 Monitor types are attached in the system 
            #while a video firmware update is performed.
            #When this problem persist, the only workaround is to unplugged the 4820 monitors to continue the update of
            #the 6149 monitors.
            if (ret == ERROR_SEGFAULT or ret == ERROR_CODE_AIP_SEGFAULT ) :
                ret = self.recover(aipflashCommand,str(self.fw_path))
                if ret != 0:
                    print "===================================================WARNING!======================================================"
                    print "\nYou may have a attached a combination of 6149 and 4820 type monitors while performing 6149 Monitor Video Firmware.\n"
                    print "Please unplugged the attached 4820 Monitors to proceed firmware update recovery of 6149 Monitors.\n"
                    TLOG.logger.info('You may have a attached a combination of 6149 and 4820 type monitors while performing 6149 Monitor Video Firmware.\n')
                    TLOG.logger.info('Please unplugged the attached 4820 Monitors to proceed firmware update recovery of 6149 Monitors.\n')
                    
                    msg = '\"Press any key to continue...\"'
                    utils.runShell('read -p  ' + msg + ' -n1 -s')

            time.sleep(10)
            
            if ret != EXIT_SUCCESS:
                TLOG.logger.info('WistronTouchFlashRecoveryTask::run() - Goes into recovery now.')
                return ret #return immediately
            
            TLOG.logger.info('WistronTouchFlashRecoveryTask::run() - Deleting AIP Recovery file.' )
            utils.runShell('rm ' + AIP_RECOVERY_FILE)
            
            #ret = utils.runSubprocess(info,'VideoFwFlashTask_6149_DWU')
        
        if ret != EXIT_SUCCESS:
            return ret #return immediately
        else:
            print '[VideoFwFlashTask_6149_DWU] - Task Completed! - Please check log file at '  + MUS_LOG_FILE_DIR         
            TLOG.logger.info('VideoFwFlashTask_6149_DWU::run() -----------------END-----------------\n')
            return ret

##==========================================================================================================
#VideoFwRecoveryFlashTask - Class for the Wistron Recovery Task for Video Flash
##==========================================================================================================        
class VideoFwRecoveryFlashTask:
    
    fw_name = ' '
    
    def __init__(self,name):
        '''
        Constructor
        '''
        self.fw_name = name
        TLOG.logger.info('WistronTouchFlashRecoveryTask Constructed')
    
          
    def run(self,fw_path):
        TLOG.logger.info('WistronTouchFlashRecoveryTask::run()-----------------START---------------')
        
        ret = utils.doAipRecover(fw_path,self.fw_name)
        print 'WistronTouchFlashRecoveryTask Done.' + '{' + str(ret) + '}'
        
        TLOG.logger.info('WistronTouchFlashRecoveryTask::run() - Deleting AIP Recovery file.' )
        utils.runShell('rm ' + AIP_RECOVERY_FILE)
        
        TLOG.logger.info('WistronTouchFlashRecoveryTask::run()-----------------END-----------------\n')
        print 'System is rebooting now...'
        utils.runShell('reboot')